#ifndef _CROSSPLANELUT_H
#define _CROSSPLANELUT_H

//#pragma once

/* Technicolor Confidential - Trade Secret
*/
/************* COPYRIGHT AND CONFIDENTIALITY INFORMATION *********
Copyright (c) 2015 Technicolor R&D France All Rights Reserved This program
contains proprietary information which is a trade secret/business secret of
Technicolor R&D France and is protected, even if unpublished, under applicable
Copyright laws (including French droit d'auteur). Recipient is to retain this
program in confidence and is not permitted to use or make copies thereof other
than as permitted in a written agreement with Technicolor R&D France unless
otherwise expressly allowed by applicable laws or by Technicolor R&D France
under express agreement.
*************************************************************************/
#include <vector>
#include <iosfwd>
#include <stdint.h>
#include "Frame.H"
#include <cassert>
#include "Global.H"
#if TCH_CROSSPLANE_CRCB

#define DYNAMIC_BETA_LUT      (float)(1<<10)


#define TCH_PW_LINEAR_INTEGER 1

namespace Tch {

struct LutUVFactor {
    std::vector<float> data;
    float linear_quantizer; // assume idx 0 is at the middle of the interval

    float operator[](float x) const { // TODO_TCH TO CHECK smell
        const int idx=(int)(x/linear_quantizer);
        if (idx<=0)                     return data[0];
        else if (x>=(int)data.size()-1) return data.back();
        const float dx=(x/linear_quantizer-(float)idx);
        const float dy=(data[idx+1]-data[idx])/linear_quantizer;
        return data[idx]+dx*dy;
    }
};

typedef LutUVFactor LutBeta0;

#if TCH_PW_LINEAR_INTEGER
struct Segment { // Atf
    float scale, delta; // hdr_recon_scale_val hdr_recon_range_val
};

struct SegmentInteger { // Atf
    uint32_t scale, delta; // hdr_recon_scale_val hdr_recon_range_val
};

struct PWMode { enum Type { Qc=0, Tch=1 }; };
struct PWModel {
    PWMode::Type mode; // 0:QC: (x-off)*scale ibnterval on y, clip01. 1:TCH: scale*x+off, no clip, interval on x
    int hdr_recon_scale_bit_depth; // 4 bits
    int hdr_recon_scale_frac_bit_depth; // 4 bits
    int hdr_recon_offset_bit_depth; // 4 bits
    int hdr_recon_offset_frac_bit_depth; // 4 bits
    int hdr_recon_num_comps_minus1; // =0
    int hdr_recon_equal_ranges_flag;
    float offset0; // hdr_recon_global_offset_val
    std::vector<Segment> lines; //  size=hdr_recon_num_ranges,

    int hdr_recon_num_ranges() const { return (int)ilines.size(); }
    uint32_t ioffset0;
    std::vector<SegmentInteger> ilines; //  size=hdr_recon_num_ranges,
};


void applyForwardATF2(Frame *inp, int component, Tch::PWModel &pw);

std::vector<float> computeLut(const Tch::PWModel &pw, int lut_size);

void  applyATF(Frame *inp, const  std::vector<float> lut);

// transform a LUT [0..1]->[0..1] into a set of PW lines
PWModel LUT2PWLinearTch(const std::vector<float> &v, const std::vector<int> &n, int nb_interval_max, float error_max0);

PWModel refineIntegerModel(const  std::vector<float> &v, const std::vector<int> &n, const PWModel &pw, PWMode::Type integer_mode=PWMode::Tch); // refine for integer model


PWModel lut2PWL(const std::vector<float> v, int nb_interval);

#if TCH_PW_LINEAR_INTEGER
namespace HDRConvertATFns {
uint32_t evaluateIntRepnQc(float x, int numBitsVal, int numBitsFrac);
uint32_t evaluateIntRepnTch(float x, int numBitsVal, int numBitsFrac);
void toFloat(Tch::PWModel &pw);
}
#endif

namespace Integer {
std::vector<int> computeLut(const Tch::PWModel &pw, int quantizerInput,int quantizerLut);
}


#endif

}
#endif
#endif //_CROSSPLANELUT_H