#ifndef _CROSSPLANERESHAPER_H
#define _CROSSPLANERESHAPER_H

//#pragma once
/* Technicolor Confidential - Trade Secret
*/

/************* COPYRIGHT AND CONFIDENTIALITY INFORMATION *********
Copyright (c) 2015 Technicolor R&D France All Rights Reserved This program
contains proprietary information which is a trade secret/business secret of
Technicolor R&D France and is protected, even if unpublished, under applicable
Copyright laws (including French droit dauteur). Recipient is to retain this
program in confidence and is not permitted to use or make copies thereof other
than as permitted in a written agreement with Technicolor R&D France unless
otherwise expressly allowed by applicable laws or by Technicolor R&D France
under express agreement.
*************************************************************************/

/*!
*************************************************************************************
* \file CrossPlaneReshaper.h
*
* \brief
*    CrossPlaneReshaper class Header
*
* \author
*     -
*
*************************************************************************************
*/
#include "CrossPlaneLut.H"
#include <utility>
#include "Global.H"
#if TCH_CROSSPLANE_CRCB

namespace Tch  {
  typedef Tch::LutBeta0 LutComputation;

  struct RGBf  {  // floating  RGB
    float R, G, B;
    float operator[](int i) const { return (i==0)?R:(i==1?G:B); }
    float &operator[](int i) { return (i==0)?R:(i==1?G:B); }
  };

  struct YUVf {  // float YUV
    float Y, U, V;
    float operator[](int i) const { return (i==0)?Y:(i==1?U:V); }
    float &operator[](int i) { return (i==0)?Y:(i==1?U:V); }
  };

  class CrossPlaneReshaper {
  public:
    CrossPlaneReshaper();
    ~CrossPlaneReshaper();
    void forward(Frame &out, const Frame &inp);
    void inverse(Frame &out, const Frame &inp);
    void process(const Frame &RGB_framein, const Frame &Reshaped_YUV_frame) ;
    void writeCbCrATFCrossPlane(char *filename, int POC, int reshaperRangeId, int reshapingMode, int *offsetArray, int numParams);
    void encode_beta0_lut();
    void setParamsTCHYATF(Tch::PWModel &pw)  ;
    void generateRangesAndOffsets(Tch::PWModel &pw, int c); 
#if ARRS_MODEL_OPTIMIZATION
    void setCrossCompLUT(float *lut, int lutSize, int chromaIndex);
    void getForwardCrossScaleLUT(int *lut, int lutSize, int chromaIndex);
#endif
  private:

    //attributes:
    std::pair<Tch::LutBeta0, Tch::LutBeta0> LUT_Beta0s;
    int  m_inverseChromaReshapingLUT[2][MAX_LUMA_RESHAPING_LUT_SIZE];
    int  m_forwardChromaReshapingLUT[2][MAX_LUMA_RESHAPING_LUT_SIZE];
    Tch::PWModel pwlU; 
    Tch::PWModel pwlV;
    int     m_betaLutDyn;
    double m_last_written_Ba;

    std::vector<float> m_betaU;
    std::vector<float> m_betaV;
    std::vector<float> m_denomU;
    std::vector<float> m_denomV;

    std::vector<Tch::RGBf> m_imageRGB_down;
    std::vector<Tch::YUVf> m_image_reshaped_420YUV;
    std::vector<int> m_nn;

    //functions
    void compute_beta0_yuv_ref( std::vector<RGBf> &orig_RGB_samples, std::vector<YUVf> &orig_YUV_samples,std::vector<int> &nn, SampleRange range);

  };

}
#endif
#endif  //_CROSSPLANERESHAPER_H