/* The copyright in this software is being made available under the BSD
 * License, included below. This software may be subject to other third party
 * and contributor rights, including patent rights, and no such rights are
 * granted under this license.
 *
 * <OWNER> = Apple Inc.
 * <ORGANIZATION> = Apple Inc.
 * <YEAR> = 2015
 *
 * Copyright (c) 2015, Apple Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *  * Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 *  * Neither the name of the <ORGANIZATION> nor the names of its contributors may
 *    be used to endorse or promote products derived from this software without
 *    specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 */

/*!
 *************************************************************************************
 * \file ScaleFilter.H
 *
 * \brief
 *    ScaleFilter class Header
 *
 * \author
 *     - Alexis Michael Tourapis         <atourapis@apple.com>
 *
 *************************************************************************************
 */


#ifndef __ScaleFilter_H__
#define __ScaleFilter_H__

#include "Global.H"
#include "Frame.H"

typedef enum {
  DF_NN  = 0,
  DF_BI  = 1,
  DF_F0  = 2,
  DF_F1  = 3,
  DF_TM  = 4,
  DF_FV  = 5, // FastVDO downsampling filters
  DF_GS  = 6,
  DF_WCS = 7,
  DF_SVC = 8,
  DF_LZW = 9,
  DF_SNW = 10,
  DF_LZ2 = 11,
  DF_SN2 = 12,
  DF_SN3 = 13,
  DF_LZ3 = 14,
  DF_LZ4 = 15,
  DF_TOTAL
} DownSamplingFilters;

typedef enum {
  UF_NN  = 0,
  UF_F0  = 1,
  UF_FV  = 2, // FastVDO upsampling filters
  UF_GS  = 3,
  UF_LS3 = 4, // Lanczos 3 upsampler
  UF_LS4 = 5, // Lanczos 4 upsampler
  UF_TM  = 6, // MPEG-2 upsampler
  UF_LS5 = 7, // Lanczos 5 upsampler
  UF_LS6 = 8, // Lanczos 6 upsampler
  UF_TOTAL
} UpSamplingFilters;


typedef enum{
  DCF_BILINEAR       = 0,
  DCF_F0             = 1,
  DCF_F1             = 2,
  DCF_MPEG2_TM5_H    = 3,
  DCF_MPEG2_TM5_V    = 4,
  DCF_FV_H           = 5, // Entries 5-6 correspond to FastVDO's filters
  DCF_FV_V           = 6,
  DCF_GS_H           = 7,
  DCF_GS_V           = 8,
  DCF_TOTAL
} DownConversionFilters;

typedef enum{
  UCF_F0       = 0,
  UCF_F1       = 1,
  UCF_F2       = 2,
  UCF_F3       = 3,
  UCF_FVVE	   = 4, //Entries 4-7 correspond to the FastVDO filters
  UCF_FVVO     = 5,
  UCF_FVHO     = 6,
  UCF_FVHE     = 7,
  UCF_GSV0     = 8,
  UCF_GSV1     = 9,
  UCF_GSH0     = 10,
  UCF_GSH1     = 11,
  UCF_LS3V0    = 12,
  UCF_LS3V1    = 13,
  UCF_LS3H0    = 14,
  UCF_LS3H1    = 15,
  UCF_LS4V0    = 16,
  UCF_LS4V1    = 17,
  UCF_LS4H0    = 18,
  UCF_LS4H1    = 19,
  UCF_TM5V0    = 20,
  UCF_TM5V1    = 21,
  UCF_TM5H0    = 22,
  UCF_TM5H1    = 23,
  UCF_TOTAL
} UpConversionFilters;

// Filters are specified as follows
// pos0 => number of taps
// pos1 = pos0+1 .. pos2=(pos1 + pos0) => tap values
// pos2 => offset
// pos3 = pos2+1 => shift  
// It is important that all phases for a filter to have the same sum/precision
static const double UCF_Filters[8][4][15] = {
  { // UF_F0
//    { 2.0,   0.0,  +64.0,                +32.0, 6.0},
//    { 4.0,  -2.0,  +16.0,  +54.0,  -4.0, +32.0, 6.0},
//    { 4.0,  -4.0,  +36.0,  +36.0,  -4.0, +32.0, 6.0},
//    { 4.0,  -4.0,  +54.0,  +16.0,  -2.0, +32.0, 6.0},  
    { 2.0,   0.0, +256.0,                +128.0, 8.0},
    { 4.0,  -8.0,  +64.0, +216.0, -16.0, +128.0, 8.0},
    { 4.0, -16.0, +144.0, +144.0, -16.0, +128.0, 8.0},
    { 4.0, -16.0, +216.0,  +64.0,  -8.0, +128.0, 8.0},  
  },
  { // UF_FV
//    { 2.0,   0.0, +32.0,                               +16.0, 5.0},
//    { 6.0,  +0.0,  -2.0,   +7.0,  +30.0,  -4.0,  +1.0, +16.0, 5.0},
//    { 4.0,  -2.0, +18.0,  +18.0,   -2.0,               +16.0, 5.0},
//    { 6.0,  +1.0,  -4.0,  +30.0,   +7.0,  -2.0,  +0.0, +16.0, 5.0},  

    { 2.0,   0.0, +256.0,                               +128.0, 8.0},
    { 6.0,  +0.0,  -16.0,  +56.0, +240.0, -32.0,  +8.0, +128.0, 8.0},
    { 4.0, -16.0, +144.0, +144.0,  -16.0,               +128.0, 8.0},
    { 6.0,  +8.0,  -32.0, +240.0,  +56.0, -16.0,  +0.0, +128.0, 8.0},  
  },
  { // UF_GS
//    { 2.0,   0.0, +128.0,                 +64.0, 7.0},
//    { 4.0,  -3.0,  +29.0, +111.0,  -9.0,  +64.0, 7.0},
 //   { 4.0,  -8.0,  +72.0,  +72.0,  -8.0,  +64.0, 7.0},
 //   { 4.0,  -9.0, +111.0,  +29.0,  -3.0,  +64.0, 7.0},  
    { 2.0,   0.0, +256.0,                 +128.0, 8.0},
    { 4.0,  -6.0,  +58.0, +222.0, -18.0,  +128.0, 8.0},
    { 4.0, -16.0, +144.0, +144.0, -16.0,  +128.0, 8.0},
    { 4.0, -18.0, +222.0,  +58.0,  -6.0,  +128.0, 8.0},  
  },
  { // UF_LS3
//    { 2.0,   0.0, +128.0,                               +64.0, 7.0},
//    { 6.0,  +1.0,  -9.0,  +35.0, +114.0, -17.0,  +4.0,  +64.0, 7.0},
//    { 6.0,  +3.0, -17.0,  +78.0,  +78.0, -17.0,  +3.0,  +64.0, 7.0},
//    { 6.0,  +4.0, -17.0, +114.0,  +35.0,  -9.0,  +1.0,  +64.0, 7.0},  
    { 2.0,   0.0, +256.0,                                +128.0, 8.0},
    { 6.0,  +2.0,  -18.0,  +70.0, +228.0, -34.0,  +8.0,  +128.0, 8.0},
    { 6.0,  +6.0,  -34.0, +156.0, +156.0, -34.0,  +6.0,  +128.0, 8.0},
    { 6.0,  +8.0,  -34.0, +228.0,  +70.0, -18.0,  +2.0,  +128.0, 8.0},  
  },
  { // UF_LS4
    { 2.0,  0.0, +256.0,                                               +128.0, 8.0},
    { 8.0, -1.0,   +8.0,  -23.0,  +72.0, +229.0, -39.0,  +14.0,  -4.0, +128.0, 8.0},
    { 8.0, -3.0,  +15.0,  -43.0, +159.0, +159.0, -43.0,  +15.0,  -3.0, +128.0, 8.0},
    { 8.0, -4.0,  +14.0,  -39.0, +229.0,  +72.0, -23.0,   +8.0,  -1.0, +128.0, 8.0},
  },
  { // UF_TM
    { 2.0,  0.0, +256.0,                               +128.0, 8.0},
    { 6.0,  +3.0, -16.0,  +67.0, +227.0, -32.0,  +7.0, +128.0, 8.0},
    { 6.0, +21.0, -52.0, +159.0, +159.0, -52.0, +21.0, +128.0, 8.0},
    { 6.0,  +7.0, -32.0, +227.0,  +67.0, -16.0,  +3.0, +128.0, 8.0},  
  },
  { // UF_LS5
    { 2.0,  0.0, +256.0,                                                           +128.0, 8.0},
    { 10.0, +1.0, -5.0,  +12.0, -27.0,  +74.0, +230.0, -41.0,  +18.0,  -8.0, +2.0, +128.0, 8.0},
    { 10.0, +2.0, -8.0,  +21.0, -47.0, +160.0, +160.0, -47.0,  +21.0,  -8.0, +2.0, +128.0, 8.0},
    { 10.0, +2.0, -8.0,  +18.0, -41.0, +230.0,  +74.0, -27.0,  +12.0,  -5.0, +1.0, +128.0, 8.0},
  },
  { // UF_LS6
    { 2.0,   0.0, +256.0,                                                                       +128.0, 8.0},
    { 12.0,  0.0,   +3.0,  -7.0, +14.0, -29.0,  +75.0, +230.0, -43.0, +20.0, -10.0, +5.0, -2.0, +128.0, 8.0},
    { 12.0, -1.0,   +5.0, -12.0, +24.0, -49.0, +161.0, +161.0, -49.0, +24.0, -12.0, +5.0, -1.0, +128.0, 8.0},
    { 12.0, -2.0,   +5.0, -10.0, +20.0, -43.0, +230.0,  +75.0, -29.0, +14.0,  -7.0, +3.0,  0.0, +128.0, 8.0},
  }
};

static const double DCF_Filters[14][2][22] = {
  { // DF_F0
    //{ 3.0,  +1.0,  +6.0,  +1.0,  +4.0, 3.0},
    //{ 2.0,  +4.0,  +4.0,         +4.0, 3.0},
    { 3.0,   +64.0,  +384.0,  +64.0,  +256.0, 9.0},
    { 2.0,  +256.0,  +256.0,          +256.0, 9.0},
  },
  { // DF_F1
   // { 3.0,  +1.0,  +2.0,  +1.0,  +2.0, 2.0},
   // { 2.0,  +2.0,  +2.0,         +2.0, 2.0},
    { 3.0,  +128.0,  +256.0,  +128.0,  +256.0, 9.0},
    { 2.0,  +256.0,  +256.0,           +256.0, 9.0},
  },
  { // DF_TM5
    {11.0,  +21.0,  +0.0, -52.0,  +0.0, +159.0, +256.0, +159.0,  +0.0, -52.0,   0.0, +21.0,       +256.0, 9.0},
    {12.0,   +5.0, +11.0, -21.0, -37.0,  +70.0, +228.0, +228.0, +70.0, -37.0, -21.0, +11.0, +5.0, +256.0, 9.0},
  },
  { // DF_FV
   // {  9.0,  +1.0,   0.0, -8.0,  +16.0, +46.0, +16.0, -8.0,  0.0, +1.0,       +32.0, 6.0},
   // { 10.0,  +1.0,   0.0, -3.0,   +6.0, +28.0, +28.0, +6.0, -3.0, +0.0, +1.0, +32.0, 6.0},
    {  9.0,  +8.0,   0.0, -64.0,  +128.0, +368.0, +128.0, -64.0,   0.0, +8.0,       +256.0, 9.0},
    { 10.0,  +8.0,   0.0, -24.0,   +48.0, +224.0, +224.0, +48.0, -24.0, +0.0, +8.0, +256.0, 9.0},
  },
  { // DF_GS
    {15.0,  -0.01716352771649 * 512, 0.0, +0.04066666714886 * 512, 0.0, -0.09154810319329 * 512, 0.0, 0.31577823859943 * 512, 0.50453345032298 * 512,  0.31577823859943 * 512, 0.0, -0.09154810319329 * 512, 0.0, 0.04066666714886 * 512, 0.0, -0.01716352771649 * 512,       +256.0, 9.0},
    {16.0, -0.00945406160902 * 512, -0.01539537217249 * 512, 0.02360533018213 * 512, 0.03519540819902 * 512, -0.05254456550808 * 512, -0.08189331229717 * 512, 0.14630826357715 * 512, 0.45417830962846 * 512, 0.45417830962846 * 512, 0.14630826357715 * 512, -0.08189331229717 * 512, -0.05254456550808 * 512, 0.03519540819902 * 512, 0.02360533018213 * 512, -0.01539537217249 * 512, -0.00945406160902 * 512  , +256.0, 9.0},
  },
  {  // DF_WCS
    {11.0, 2.0,    -3.0,    -9.0,     6.0,    39.0,    58.0,    39.0,     6.0,    -9.0,    -3.0,     2.0,     +64.0, 7.0},
    {12.0, 1.0,     0.0,    -7.0,    -5.0,    22.0,    53.0,    53.0,    22.0,    -5.0,    -7.0,     0.0,     1.0, +64.0, 7.0}
  },
  { // DF_SVC
    {11.0, 0.016512788 * 256,  -9.77064E-18 * 256, -0.075189625 * 256, 1.69232E-17 * 256, 0.308132812 * 256, 0.50108805 * 256,  0.308132812 * 256,  1.69232E-17 * 256,  -0.075189625 * 256,  -9.77064E-18 * 256, 0.016512788 * 256, 128.0, 8.0 },
    {12.0, 0.005353954 * 256, 0.019185222 * 256, -0.039239076 * 256, -0.071592303 * 256, 0.138951671 * 256, 0.447340531 * 256, 0.447340531 * 256,  0.138951671 * 256, -0.071592303 * 256, -0.039239076 * 256, 0.019185222 * 256, 0.005353954 * 256, 128.0, 8.0}
  },
  { // DF_LZW
    {15.0, 0.00190089262242 * 2048,   0.01183528116391 * 2048,    0.00000000000000 * 2048,    -0.04505642829799 * 2048,    -0.04734112465563 * 2048,    0.08323159585050 * 2048,    0.29483273086709 * 2048,    0.40119410489940 * 2048,    0.29483273086709 * 2048,   0.08323159585050 * 2048, -0.04734112465563 * 2048,    -0.04505642829799 * 2048,    0.00000000000000 * 2048,    0.01183528116391 * 2048,    0.00190089262242 * 2048, 1024, 11},
    { 14.0,  0.00697781694123 * 2048,      0.01100508557770 * 2048,      -0.02103907505369 * 2048,      -0.05884522739626 * 2048,      0.00000000000000 * 2048,      0.18935167548323 * 2048,      0.37254972444780 * 2048,      0.37254972444780 * 2048,      0.18935167548323 * 2048,      0.00000000000000 * 2048,      -0.05884522739626 * 2048,      -0.02103907505369 * 2048,      0.01100508557770 * 2048,      0.00697781694123 * 2048, 1024.0, 11.0}
  },
  { // DF_SNW
    {13.0,    0.00571729514027 * 128.0,      0.01791720928626 * 128.0,      -0.02500536578022 * 128.0,      -0.07308632957148 * 128.0,      0.03993394132849 * 128.0,      0.30679811688568 * 128.0,      0.45545026542200 * 128.0,      0.30679811688568 * 128.0,      0.03993394132849 * 128.0,      -0.07308632957148 * 128.0,      -0.02500536578022 * 128.0,      0.01791720928626 * 128.0,      0.00571729514027 * 128.0,      64.0, 7.0},
    { 14.0, 0.00016629082500 * 256.0,      0.01501862112403 * 256.0,      0.00483721502879 * 256.0,      -0.05885266302048 * 256.0,      -0.04391848267708 * 256.0,      0.16770594201608 * 256.0,      0.41504307670367 * 256.0,     0.41504307670367 * 256.0,      0.16770594201608 * 256.0,     -0.04391848267708 * 256.0,     -0.05885266302048 * 256.0,      0.00483721502879 * 256.0,      0.01501862112403 * 256.0,      0.00016629082500 * 256.0, 128.0, 8.0 }
  },
  {  // DF_LZ2
    {15.0,
    -0.00632274800093 * 256,
    0.00000000000000 * 256,
    0.02991834939159 * 256,
    0.00000000000000 * 256,
    -0.08310652608775 * 256,
    0.00000000000000 * 256,
    0.30981465204533 * 256,
    0.49939254530351 * 256,
    0.30981465204533 * 256,
    0.00000000000000 * 256,
    -0.08310652608775 * 256,
    0.00000000000000 * 256,
    0.02991834939159 * 256,
    0.00000000000000 * 256,
    -0.00632274800093 * 256, 128.0, 8.0},
    {16.0,
      -0.00198530798042 * 512.0,
      -0.00752708715723 * 512.0,
      0.01573387488128 * 512.0,
      0.02772449226106 * 512.0,
      -0.04583028312543 * 512.0,
      -0.07615195442538 * 512.0,
      0.14134196995234 * 512.0,
      0.44669429559377 * 512.0,
      0.44669429559377 * 512.0,
      0.14134196995234 * 512.0,
      -0.07615195442538 * 512.0,
      -0.04583028312543 * 512.0,
      0.02772449226106 * 512.0,
      0.01573387488128 * 512.0,
      -0.00752708715723 * 512.0,
      -0.00198530798042 * 512.0, 256.0, 9.0 }
    
  },
  { // DF_SN2
    { 15.0,
      -0.00886299106559 * 512.0,
      0.00000000000000 * 512.0,
      0.03533552841036 * 512.0,
      0.00000000000000 * 512.0,
      -0.08813892574297 * 512.0,
      0.00000000000000 * 512.0,
      0.31190085501694 * 512.0,
      0.49953106676252 * 512.0,
      0.31190085501694 * 512.0,
      0.00000000000000 * 512.0,
      -0.08813892574297 * 512.0,
      0.00000000000000 * 512.0,
      0.03533552841036 * 512.0,
      0.00000000000000 * 512.0,
      -0.00886299106559 * 512.0, 256.0, 9.0},
    { 16.0, 
      -0.00293861597778 * 256.0,
      -0.01004182911844 * 256.0,
      0.01927196301938 * 256.0,
      0.03169918776063 * 256.0,
      -0.04966144977164 * 256.0,
      -0.07932167506148 * 256.0,
      0.14344838825693 * 256.0,
      0.44754403089239 * 256.0,
      0.44754403089239 * 256.0,
      0.14344838825693 * 256.0,
      -0.07932167506148 * 256.0,
      -0.04966144977164 * 256.0,
      0.03169918776063 * 256.0,
      0.01927196301938 * 256.0,
      -0.01004182911844 * 256.0,
      -0.00293861597778 * 256.0, 128.0, 8.0}
  },
  { // DF_SN3
    { 15.0,
      -0.00067732252889 * 1024.0,
      -0.01187238377794 * 1024.0,
      0.01886537347052 * 1024.0,
      0.03282746274537 * 1024.0,
      -0.07278440675580 * 1024.0,
      -0.04925606519713 * 1024.0,
      0.30555914564961 * 1024.0,
      0.55467639278853 * 1024.0,
      0.30555914564961 * 1024.0,
      -0.04925606519713 * 1024.0,
      -0.07278440675580 * 1024.0,
      0.03282746274537 * 1024.0,
      0.01886537347052 * 1024.0,
      -0.01187238377794 * 1024.0,
      -0.00067732252889 * 1024.0, 512.0, 10.0},
    {14.0,
      -0.00699733029589 * 256.0,
      -0.00364087058553 * 256.0,
      0.03928171002786 * 256.0,
      -0.01140308043259 * 256.0,
      -0.10224241186895 * 256.0,
      0.10042934057489 * 256.0,
      0.48457264258020 * 256.0,
      0.48457264258020 * 256.0,
      0.10042934057490 * 256.0,
      -0.10224241186895 * 256.0,
      -0.01140308043259 * 256.0,
      0.03928171002786 * 256.0,
      -0.00364087058553 * 256.0,
      -0.00699733029589 * 256.0,
      128.0, 8.0}
  },
  { // DF_LZ3
    { 17.0,
      0.00480212008232 * 256.0,
      -0.00409385963083 * 256.0,
      -0.01900940873498 * 256.0,
      0.02310408604406 * 256.0,
      0.03610013444384 * 256.0,
      -0.07603763493990 * 256.0,
      -0.05014978047763 * 256.0,
      0.30733568526821 * 256.0,
      0.55589731588983 * 256.0,
      0.30733568526821 * 256.0,
      -0.05014978047763 * 256.0,
      -0.07603763493990 * 256.0,
      0.03610013444384 * 256.0,
      0.02310408604406 * 256.0,
      -0.01900940873498 * 256.0,
      -0.00409385963083 * 256.0,
      0.00480212008232 * 256.0, 128.0, 8.0},
    {18.0, 
      0.00168005324555 * 256.0,
      0.00405559129955 * 256.0,
      -0.01554711355088 * 256.0,
      -0.00492229194265 * 256.0,
      0.04506212555057 * 256.0,
      -0.01215504745022 * 256.0,
      -0.10509848760398 * 256.0,
      0.10138978248878 * 256.0,
      0.48553538796329 * 256.0,
      0.48553538796329 * 256.0,
      0.10138978248878 * 256.0,
      -0.10509848760398 * 256.0,
      -0.01215504745022 * 256.0,
      0.04506212555057 * 256.0,
      -0.00492229194265 * 256.0,
      -0.01554711355088 * 256.0,
      0.00405559129955 * 256.0,
      0.00168005324555 * 256.0, 128.0, 8.0}
  },
  {  // DF_LZ4
    {13.0,
    -0.00832391833024 * 1024.0,
    0.00961238047504 * 1024.0,
    0.03374816073530 * 1024.0,
    -0.05999673019609 * 1024.0,
    -0.06007737796902 * 1024.0,
    0.29966105988877 * 1024.0,
    0.57075285079248 * 1024.0,
    0.29966105988877 * 1024.0,
    -0.06007737796902 * 1024.0,
    -0.05999673019609 * 1024.0,
    0.03374816073530 * 1024.0,
    0.00961238047504 * 1024.0,
    -0.00832391833024 * 1024.0, 512.0, 10.0},
  {14.0,
    -0.00291663662971 * 1024.0,
    -0.00633435442913 * 1024.0,
    0.03072431945341 * 1024.0,
    0.00000000000000 * 1024.0,
    -0.09954679502905 * 1024.0,
    0.08516187621385 * 1024.0,
    0.49291159042062 * 1024.0,
    0.49291159042062 * 1024.0,
    0.08516187621385 * 1024.0,
    -0.09954679502905 * 1024.0,
    0.00000000000000 * 1024.0,
    0.03072431945341 * 1024.0,
    -0.00633435442913 * 1024.0,
    -0.00291663662971 * 1024.0, 512.0, 10.0}
  }
};



class ScaleFilter {
public:
  int     m_numberOfTaps;
  int     m_positionOffset;
  float  *m_floatFilter;
  float   m_floatOffset;
  float   m_floatScale;
  int32  *m_i32Filter;
  int32   m_i32Offset;
  int32   m_i32Shift;
  bool    m_clip;
  ScaleFilter(int filter, int mode, int paramsMode, int inOffset, int inShift, int *outOffset, int *outShift, int phase);
  ~ScaleFilter();
};

#endif
