
#include <vector>
#include <iostream>
#include <fstream>
#include <algorithm>
#include <cmath>
#include <string>
#include "Global.H"
#include "CrossPlaneLut.H" // because of external inc
#if TCH_CROSSPLANE_CRCB
using namespace std;
using namespace Tch;

#if TCH_PW_LINEAR_INTEGER

namespace {


  // ////////////////////////////////////////////////////////////////////////////////////////////////////
  // PWL FITTER
  // first point as a hard constraint
  struct Line { float coefs[2]; };
  struct Segments {
    std::vector<int> pivot;
    std::vector<Line> lines;
  };

  float clip01(float x) { return (x<=0.f)?0.f:(x>=1.f?1.f:x); }
  const float kEpsSlope=1e-4f;
  const double kMaxError=1.5;
  const float kEpsLut=1e-6f; // consider 2 elts == in a LUT
  const float kMarginPct=0.0f; // 0: no smoothing
  const float kErrorModel=1e-3f;
  const float kErrorGrowingFactor=1.666666f;
  const double kEpsDeterminant=1e-12;
  const int kMinNbPtsForSegment=32;
  struct Point { float x,y; int n;};

  double eval(const Line p,double x) {  return (float)((double)p.coefs[0]+x*p.coefs[1]); }

  bool fitLineC0(Point s0,const vector<Point> &v,Line &p) {
    // least square AX=B w.r.t AX0=Y0
    double AtA=0.;
    double AtB=0.;
    int nn=0;
    for(int k=1;k<(int)v.size();++k) {
      const double dx=v[k].x-s0.x;
      const double dy=v[k].y-s0.y;
      AtA+=v[k].n*pow(dx,2.);
      AtB+=v[k].n*dx*dy;
      nn+=v[k].n;
    }
    if (nn<kMinNbPtsForSegment) return false;
    if (fabs(AtA)<kEpsDeterminant) return false;
    p.coefs[1]=(float)(AtB/AtA);
    p.coefs[0]=s0.y-p.coefs[1]*s0.x;
    return true;
  }

  double error(const Line &p,const std::vector<Point> &v) {
     double s=0.;
    double maxi=0.;
    for(int k=0;k<(int)v.size();++k) {
      const double e=fabs(eval(p,v[k].x)-v[k].y);
      s+=e;
      maxi=std::max(maxi,e);
    }
    return s;
   }

  vector<Point> extractPoints(const std::vector<Point> &v,int i0,int i1,int ilast) {
    vector<Point> v2;
    int size=i1-i0+1;
    size+=(int)((float)size*kMarginPct+0.5f);
    if (i0+size>=ilast) size=ilast-i0+1;
if(!(size>=2))
    assert(size>=2);
    v2.reserve(size);
    for(int k=0;k<size;++k) {
      const int ii=i0+k;
      assert(ii>=0&&ii<(int)v.size());
      v2.push_back(v[ii]);
    }
    return v2;
  }

 //////////////////////////////////////////////////////////////////////////////////////////////////////
 //////////////////////////// COVNERTER PWlines -> ATF model /////////////////////////////////////////////////////////
 //////////////////////////////////////////////////////////////////////////////////////////////////////
  vector<float> getDelta(const Segments &s,int lut_size) {
    assert(s.pivot.front()==0);
    vector<float> v(s.pivot.size()-1);
    for(int i=1;i<(int)s.pivot.size();++i) {
      //        if (mode==PWMode::Qc) {
      //            float x0=s.pivot[i-1]/(float)(lut_size-1);
      //            float x1=s.pivot[i]/(float)(lut_size-1);
      //            float y0=eval(s.lines[i],x0);
      //            float y1=eval(s.lines[i],x1);
      //            v[i-1]=y1-y0;
      //        } else {
      v[i-1]=(float)(s.pivot[i]-s.pivot[i-1])/(float)(lut_size-1);
      // }
    }
    return v;
  }

  vector<float> getScale(const Segments &s) {
    vector<float> v(s.lines.size());
    for(int i=0;i<(int)s.lines.size();++i) {
      //        if (mode==PWMode::Qc) {
      //            if (fabs(s.lines[i].coefs[1])<kEpsSlope) v[i]=1.f/kEpsSlope;
      //            else                                     v[i]=1.f/s.lines[i].coefs[1];
      //        } else
      v[i]=s.lines[i].coefs[1];
    }
    return v;
  }

  float getOffset0(const Segments &s) {
#if 0
    if (mode==PWMode::Qc) { // mode QC: (x-delta)*scale
      const double delta0=(s.pivot[1]-s.pivot[0])/(float)(lut_size-1);
      return -(s.lines[0].coefs[0]+delta0)/s.lines[0].coefs[1]+delta0;
    } else if (mode==PWMode::Tch) { // mode tch: scale*x+b
      return s.lines[0].coefs[0];
    }
#endif
    // assume offset0 = 0 for Qc mode... TODO
    return s.lines[0].coefs[0];
    assert(false);
    return 0.f;
  }

  vector<Segment> getSegments(const Segments &s,int lut_size) {
    vector<float> v1=getDelta(s,lut_size);
    vector<float> v2=getScale(s);
    vector<Segment> v3(v1.size());
    for(int k=0;k<(int)v3.size();++k) { v3[k].delta=v1[k]; v3[k].scale=v2[k]; }
    return v3;
  }

  Tch::PWModel segmentsToModel(const Segments &s,int lut_size) {
    PWModel pw;
    pw.offset0=getOffset0(s);
    pw.lines=getSegments(s,lut_size);
    pw.hdr_recon_num_comps_minus1=0;
    pw.hdr_recon_equal_ranges_flag=0;
    pw.mode=PWMode::Tch; //default
    return pw;
  }

  vector<float> getAtfRangeIn(const PWModel &pw) {
    vector<float> v(pw.lines.size()+1);
    v[0]=0.;
    for(int k=1;k<(int)v.size();++k) v[k]=v[k-1]+pw.lines[k-1].delta;
    return v;
  }

  vector<float> getAtfRangeOut(const PWModel &pw) {
    vector<float> v(pw.lines.size()+1);
    //    if (pw.mode==PWMode::Qc) {
    //        v[0]=-pw.offset0*pw.lines[0].scale;
    //    } else if (pw.mode==PWMode::Tch) {
    v[0]=pw.offset0;
    //    } else assert(false);
    for(int k=1;k<(int)v.size();++k) v[k]=v[k-1]+pw.lines[k-1].delta*pw.lines[k-1].scale;
    return v;
  }

  vector<float> getAtfOffset2(const PWModel &pw) {
    const vector<float> In=getAtfRangeIn(pw);
    const vector<float> Out=getAtfRangeOut(pw);
    vector<float> v(pw.lines.size());
    for(int k=0;k<(int)v.size();++k) {
      //        if (pw.mode==PWMode::Qc) {
      //            v[k]=In[k+1]-Out[k+1]/pw.lines[k].scale;
      //        } else if (pw.mode==PWMode::Tch) {
      v[k]=Out[k+1]-In[k+1]*pw.lines[k].scale;
      //        }
    }
    return v;
  }

  float slopeConstraint(float x) {
    if (fabs(x)<kEpsSlope) {
      if (x<0.f) return -kEpsSlope;
      else       return kEpsSlope;
    } else return x;
  }

  //////////////////////////////////////////////////////////////////////////////////////////////////////

  Segments modelToSegments(const Tch::PWModel &pw,int lut_size) {
    Segments s;
    s.pivot.push_back(0); // by construction
    assert(pw.mode==PWMode::Tch);
    // dirty trick to merge intervals

    const vector<float> pOffset = getAtfOffset2(pw);
    for(int k=0;k<(int)pw.lines.size();++k) {
      double deltaf=0.;
      int deltai=0;
      int k1=k;
      while(k1<(int)pw.lines.size()&&deltai<=2) {
        deltaf+=pw.lines[k1].delta;
        deltai=(int)(deltaf*(float)(lut_size-1)+0.5f);
        ++k1;
      }
      s.pivot.push_back((int)(s.pivot.back()+deltai));
      Line line;
      line.coefs[0]=pOffset[k];
      line.coefs[1]=pw.lines[k].scale;
      s.lines.push_back(line);
      k=k1-1;
    }
    return s;
  }
}

namespace  {

  void initFields(Tch::PWModel &pw2,const Tch::PWModel &pw) {
    pw2.hdr_recon_equal_ranges_flag    =pw.hdr_recon_equal_ranges_flag;
    pw2.hdr_recon_num_comps_minus1     =pw.hdr_recon_num_comps_minus1;
    pw2.hdr_recon_offset_bit_depth     =pw.hdr_recon_offset_bit_depth;
    pw2.hdr_recon_offset_frac_bit_depth=pw.hdr_recon_offset_frac_bit_depth;
    pw2.hdr_recon_scale_bit_depth      =pw.hdr_recon_scale_bit_depth;
    pw2.hdr_recon_scale_frac_bit_depth =pw.hdr_recon_scale_frac_bit_depth;
    pw2.mode=pw.mode;
  }

#if TCH_PW_LINEAR_INTEGER
  void toInteger(Tch::PWModel &pw) {
    if (pw.mode==PWMode::Qc) {
      pw.ioffset0=Tch::HDRConvertATFns::evaluateIntRepnQc(pw.offset0,pw.hdr_recon_offset_bit_depth+pw.hdr_recon_offset_frac_bit_depth,pw.hdr_recon_offset_frac_bit_depth);
      pw.ilines.resize(pw.lines.size());
      for (int i = 0; i<(int)pw.lines.size(); ++i) {
        pw.ilines[i].delta=Tch::HDRConvertATFns::evaluateIntRepnQc(pw.lines[i].delta,pw.hdr_recon_offset_bit_depth+pw.hdr_recon_offset_frac_bit_depth,pw.hdr_recon_offset_frac_bit_depth);
        pw.ilines[i].scale=Tch::HDRConvertATFns::evaluateIntRepnQc(pw.lines[i].scale,pw.hdr_recon_scale_bit_depth+pw.hdr_recon_scale_frac_bit_depth,pw.hdr_recon_scale_frac_bit_depth);
      }
    } else {
      pw.ioffset0=HDRConvertATFns::evaluateIntRepnTch(pw.offset0,pw.hdr_recon_offset_bit_depth+pw.hdr_recon_offset_frac_bit_depth,pw.hdr_recon_offset_frac_bit_depth);
      pw.ilines.resize(pw.lines.size());
      for (int i = 0; i<(int)pw.lines.size(); ++i) {
        pw.ilines[i].delta=Tch::HDRConvertATFns::evaluateIntRepnTch(pw.lines[i].delta,pw.hdr_recon_offset_bit_depth+pw.hdr_recon_offset_frac_bit_depth,pw.hdr_recon_offset_frac_bit_depth);
        pw.ilines[i].scale=Tch::HDRConvertATFns::evaluateIntRepnTch(pw.lines[i].scale,pw.hdr_recon_scale_bit_depth+pw.hdr_recon_scale_frac_bit_depth,pw.hdr_recon_scale_frac_bit_depth);
      }
    }
  }

  Segments roundTrip(const Segments &s,const Tch::PWModel &pw,int lut_size,PWMode::Type mode_integer_model) {
    Tch::PWModel pw2=segmentsToModel(s,lut_size); // seg -> pwl
    initFields(pw2,pw);

    // encode decode
    if (mode_integer_model==PWMode::Qc) {
      // here assume a forward model and offset0=0 -> just change scale
#if TCH_PWMODEL_ATF_DOMAIN1
      for(int k=0;k<(int)pw2.lines.size();++k) pw2.lines[k].scale=pw2.lines[k].scale; // domainId=1: code scale, domainId=10 code 1/scale
#else
      for(int k=0;k<(int)pw2.lines.size();++k) pw2.lines[k].scale=1.f/pw2.lines[k].scale; // domainId=1: code scale, domainId=10 code 1/scale
#endif
    }
    toInteger(pw2);
    Tch::HDRConvertATFns::toFloat(pw2);
    if (mode_integer_model==PWMode::Qc) {
      // here assume a forward model and offset0=0 -> just change scale
#if TCH_PWMODEL_ATF_DOMAIN1
      for(int k=0;k<(int)pw2.lines.size();++k) pw2.lines[k].scale=pw2.lines[k].scale; // domainId=1: code scale, domainId=10 code 1/scale
#else
      for(int k=0;k<(int)pw2.lines.size();++k) pw2.lines[k].scale=1.f/pw2.lines[k].scale; // domainId=1: code scale, domainId=10 code 1/scale
#endif
    }
    Segments s2=modelToSegments(pw2,lut_size); // go back
    // display(s2);
    return s2;
  }
#endif
}

namespace Tch {
  namespace HDRConvertATFns { // from HDRConvertATF: to unified later when common format decided
    float evaluateFloatRepnQc(uint32_t x, int const numBitsVal, int const numBitsFrac)
    {
      assert(x < (1u << numBitsVal));
      int signPart = x >> numBitsFrac;
      int numSignBits = numBitsVal - numBitsFrac;
      assert(signPart >= 0 && signPart < (1 << numSignBits));
      (void)numSignBits;

      int fracPart = x - (signPart << numBitsFrac);
      assert(fracPart >= 0 && fracPart < (1 << numBitsFrac));

      float retVal = (float)signPart + (float)fracPart / (float)(1 << numBitsFrac);

      assert(retVal >= 0 && retVal < (1 << numBitsVal));
      return retVal;
    }

    uint32_t evaluateIntRepnQc(float x, int numBitsVal, int numBitsFrac)
    {
      if (x<0.) {
      x=0.;
      }
      float fracVal = x -(float)((int)x);
      assert(fracVal >= 0 && fracVal < 1);
      int numBitsSignif = numBitsVal - numBitsFrac;

      if (numBitsSignif == 0)
      {
        assert(x < 1 && (int)(x - fracVal) == 0);
      }
      else
      {
        const int maxi=(1 << numBitsSignif)-1;
        if (x - fracVal > maxi) {
          x=(float)maxi+fracVal;
        }
      }

      int intRepnOfFrac = (int)floor((fracVal * (float)(1 << numBitsFrac)) + 0.5f); // / (1 << numBitsFrac);
      uint32_t retVal = (int)x * (1 << numBitsFrac) + intRepnOfFrac;
      assert(retVal < (1u << numBitsVal));
      return retVal;
    }


    float evaluateFloatRepnTch(uint32_t x, int const numBitsVal, int const numBitsFrac)
    {
      assert(x < (1u << numBitsVal) );
      const int signMask=(1<<(numBitsVal-1));
      bool neg=false;
      if (x&signMask) { neg=true; x=(x&(signMask-1)); }

      int intPart = x >> numBitsFrac;
      int fracPart = x - (intPart << numBitsFrac);
      float retVal = (float)intPart + (float)fracPart / (float)(1 << numBitsFrac);
      if (neg) retVal=-retVal;
      return retVal;
    }

    uint32_t evaluateIntRepnTch(float x, int numBitsVal, int numBitsFrac)
    {
      int nbInt=numBitsVal-numBitsFrac;
      if (nbInt==0) nbInt=1;
      --nbInt; // sign
      int maxi=(1<<nbInt)-1;
      if (maxi<=0) maxi=1;
      if (x>maxi) {
        x=(float)maxi;
      }
      if (x<-maxi) {
        x=(float)-maxi;
      }
      float fracVal;
      int signMask=0;
      if (x<0.f) { fracVal=-(x-(float)((int)x)); signMask=(1<<(numBitsVal-1)); x = -x; }   // always code the sign
      else         fracVal=x - (float)((int)x);
      int intRepnOfFrac = (int)((fracVal * (float)(1 << numBitsFrac)) + 0.5f);
      //intRepnOfFrac=std::min(intRepnOfFrac,(1<<numBitsFrac)-1);
      int intPart=(int)x;
      uint32_t retVal = (intPart<< numBitsFrac) + intRepnOfFrac;

      retVal|=signMask;
      assert(retVal < (1u << numBitsVal));
      return retVal;
    }

    void toFloat(Tch::PWModel &pw) {
      if (pw.mode==PWMode::Qc) {
        pw.offset0=HDRConvertATFns::evaluateFloatRepnQc(pw.ioffset0,pw.hdr_recon_offset_bit_depth+pw.hdr_recon_offset_frac_bit_depth,pw.hdr_recon_offset_frac_bit_depth);
        pw.lines.resize(pw.ilines.size());
        for (int i = 0; i<(int)pw.lines.size(); ++i) {
          pw.lines[i].delta=HDRConvertATFns::evaluateFloatRepnQc(pw.ilines[i].delta,pw.hdr_recon_offset_bit_depth+pw.hdr_recon_offset_frac_bit_depth,pw.hdr_recon_offset_frac_bit_depth);
          pw.lines[i].scale=HDRConvertATFns::evaluateFloatRepnQc(pw.ilines[i].scale,pw.hdr_recon_scale_bit_depth+pw.hdr_recon_scale_frac_bit_depth,pw.hdr_recon_scale_frac_bit_depth);
        }
      } else {
        pw.offset0=HDRConvertATFns::evaluateFloatRepnTch(pw.ioffset0,pw.hdr_recon_offset_bit_depth+pw.hdr_recon_offset_frac_bit_depth,pw.hdr_recon_offset_frac_bit_depth);
        pw.lines.resize(pw.ilines.size());
        for (int i = 0; i<(int)pw.lines.size(); ++i) {
          pw.lines[i].delta=HDRConvertATFns::evaluateFloatRepnQc(pw.ilines[i].delta,pw.hdr_recon_offset_bit_depth+pw.hdr_recon_offset_frac_bit_depth,pw.hdr_recon_offset_frac_bit_depth);
          pw.lines[i].scale=HDRConvertATFns::evaluateFloatRepnTch(pw.ilines[i].scale,pw.hdr_recon_scale_bit_depth+pw.hdr_recon_scale_frac_bit_depth,pw.hdr_recon_scale_frac_bit_depth);
        }
      }
    }
  }

  void  applyATF(Frame *inp, const  std::vector<float> lut)
  {
    Frame* out = inp;
    float in_value;
    float coef;
    int32 index;

    for (int i = 0; i < inp->m_compSize[0]; i++) 
    {
      in_value = inp->m_floatComp[0][i];
      index = (int32) min(1023,(int)(in_value*1023.+0.5));
      coef = lut[index];
      out->m_floatComp[0][i] =  coef;
    }
  }

  void  applyChromaCorrection(Frame *inp, const  std::vector<float> lut)
  {
    Frame* out = inp;
    float in_value;
    float coef;
    int32 index;

    for (int i = 0; i < inp->m_compSize[0]; i++) 
    {
      in_value = inp->m_floatComp[0][i];
      index = (int32) min(1023,(int)(in_value*1023.+0.5));
      coef = lut[index];
      out->m_floatComp[0][i] =  coef;
    }

  }


  std::vector<float> computeLut(const Tch::PWModel &pw, int lut_size) {
    assert(pw.mode==PWMode::Tch);
    const int numRanges = (int)pw.lines.size(); // getAtfNumberRanges();
    const vector<float> pOffset = getAtfOffset2(pw);
    const vector<float> pAtfRange = getAtfRangeIn(pw); // (pw.mode==PWMode::Qc)?getAtfRangeOut(pw):
    vector<float> v(lut_size);

    for (int i = 0; i < lut_size; i++) {
      bool skip = 0;
      const float x=(float)i/(float)(lut_size-1);
      for (int j = 0; j < numRanges && skip == 0; j++)
      {
        if (x <= pAtfRange[j + 1])
        {
          // v[i] = x/pw.lines[j].scale + pOffset[j];
          //                if (pw.mode==Tch::PWMode::Qc) {
          //                    v[i] = ((x- pOffset[j]) * /*pScale*/pw.lines[j].scale);
          //                    v[i]=clip01(v[i]);
          //                } else if (pw.mode==Tch::PWMode::Tch) {
          v[i] = x *pw.lines[j].scale+pOffset[j];
          // }
          skip = 1;
        }
      }
      if (skip == 0) {

        //            if (pw.mode==Tch::PWMode::Qc) {
        //                v[i] = ((pAtfRange[numRanges] - pOffset[numRanges - 1]) * pw.lines[numRanges - 1].scale); // pScale[component][numRanges - 1];
        //                v[i]=clip01(v[i]);
        //            } else if (pw.mode==Tch::PWMode::Tch) {
        v[i] = pAtfRange[numRanges] * pw.lines[numRanges - 1].scale+pOffset[numRanges - 1];
        //}
      }
    }
    return v;
  }

  // note to optimizer: use the previous parametrization (pivot, ax+b) and then transform into the new one (scale delta).
  // to clean by estimating directly the new prms
  PWModel LUT2PWLinearTch(const vector<float> &v,const vector<int> &n,int nb_interval_max,float error_max0) {

    const bool noSlope0=false;
    // first find the start pivot
    int i0=1;
    while(i0<(int)v.size()&&fabs(v.front()-v[i0])<kEpsLut) ++i0;
    --i0;
    // last pivot
    int ilast=(int)v.size()-2;
    while(ilast>=0&&fabs(v.back()-v[ilast])<kEpsLut) --ilast;
    ++ilast;

    if (i0 > ilast) // avoid one single segment
    {
      i0 = ilast = (int)v.size() / 2;
    }

    vector<Point> pts(v.size());
    for(int k=0;k<(int)v.size();++k) {
      pts[k].x=(float)k/(float)(v.size()-1);
      pts[k].y=v[k];
      if (!n.empty()) pts[k].n=n[k]; else pts[k].n=64;
    }
    static const int kMinInterval=3;
    int nb_interval=nb_interval_max+1;
    float error_max=error_max0;
    Segments segs;
    // loop until we find a partition of nb_interval_max interval
    while(nb_interval>nb_interval_max&&error_max<kMaxError /* guard */) {
      segs.pivot.clear();
      segs.lines.clear();

      int ii=i0;
      Point s0;

      if (i0>0) {   // hard code the first pivot: flat segment
        segs.pivot.push_back(0);
        if (noSlope0) { // slope 0 forbidden: we encode the first segment with the minimal slope
          const Line line={{v[0],kEpsSlope}};
          segs.lines.push_back(line);
        } else { // just a flat segment
          const Line line={{v[0],0.f}};
          segs.lines.push_back(line);
        }
        s0.x=(float)i0/(float)(v.size()-1);
        s0.y=(float)eval(segs.lines.back(),s0.x);
      } else {
        s0=pts[ii];
      }
      segs.pivot.push_back(ii);

      while(ii<ilast&&(int)segs.lines.size()<=nb_interval_max) { // loop until we fill all intervals
        if (ii+kMinInterval>=ilast) break;
        int i;
        // loop until error is too large
        for(i=ii+kMinInterval;i<=ilast;++i) {
          vector<Point> v_pts=extractPoints(pts,ii,i,ilast);
          Line line;
          if (fitLineC0(s0,v_pts,line)) {
            if (noSlope0) line.coefs[1]=slopeConstraint(line.coefs[1]);
            if (error(line,v_pts)>error_max) break;
          }
        }
        if (i+kMinInterval>=ilast) i=ilast; // close to end, we finish
        segs.pivot.push_back(i);
        Line line;
        if (fitLineC0(s0,extractPoints(pts,ii,i,ilast),line)) {
          if (noSlope0) line.coefs[1]=slopeConstraint(line.coefs[1]);
          segs.lines.push_back(line);
          s0.x=(float)i/(float)(v.size()-1);
          s0.y=(float)eval(line,s0.x);
        } else {
          cerr<<"WARNING: bad matrix conditionning"<<endl;
        }
        ii=i;
      }
      nb_interval=(int)segs.lines.size();
      error_max*=kErrorGrowingFactor;
    }
    if (error_max>=kMaxError&&nb_interval>nb_interval_max) { // redo with less interval
      segs.pivot.pop_back();
      const int ii=segs.pivot.back();
      Point s0;
      s0.x=(float)ii/((float)v.size()-1);
      s0.y=(float)eval(segs.lines.back(),s0.x);
      segs.pivot.push_back(ilast);
      segs.lines.pop_back();
      Line line;
      if (fitLineC0(s0,extractPoints(pts,ii,ilast,ilast),line)) {
        if (noSlope0) line.coefs[1]=slopeConstraint(line.coefs[1]);
        segs.lines.push_back(line);
      } else {
        cerr<<"WARNING: bad matrix conditionning"<<endl;
      }
    }

    return segmentsToModel(segs,(int)v.size()); // transform into atf format
  }

#if TCH_PW_LINEAR_INTEGER
  // same optiz as before but encode/decode coefs after transfo
  // note to optimizer: full of useless op because we use only high level fct, to clean in a self contained simple loop
  PWModel refineIntegerModel(const vector<float> &v,const vector<int> &n,const PWModel &pw,PWMode::Type integer_mode) { // close loop optiz

    Segments s=modelToSegments(pw,(int)v.size());

    s=roundTrip(s,pw,(int)v.size(),integer_mode); // perform a first round trip in order to have the intervals quantized

    // display(s);
    Segments snew;
    vector<Point> pts(v.size());
    for(int k=0;k<(int)v.size();++k) {
      pts[k].x=(float)k/(float)(v.size()-1);
      pts[k].y=v[k];
      if (!n.empty()) pts[k].n=n[k]; else pts[k].n=64;
    }
    // last pivot
    int ilast=(int)v.size()-2;
    while(ilast>=0&&fabs(v.back()-v[ilast])<kEpsLut) --ilast;
    ++ilast;

    snew.pivot.push_back(s.pivot.front());
    snew.lines.push_back(s.lines.front()); // same as org
    for(int k=1;k<(int)s.lines.size();++k) { // first line coded as is
      snew.pivot.push_back(s.pivot[k]);
      Segments ntemp=roundTrip(snew,pw,(int)v.size(),integer_mode); // enc dec the segments
      Point s0; // take the last pt from encdec as the new start
      s0.x=(float)s.pivot[k]/((float)v.size()-1.f);
      s0.y=(float)eval(ntemp.lines[k-1],s0.x);
       vector<Point> v_pts=extractPoints(pts,s.pivot[k],s.pivot[k+1],ilast);
      Line line;
      if (!fitLineC0(s0,v_pts,line)) {
        std::cerr<<"## WARNING: bad reconvergence"<<std::endl;
      }
      snew.lines.push_back(line);
    }
    snew.pivot.push_back(s.pivot.back());

    Tch::PWModel pw2=segmentsToModel(snew,(int)v.size()); // seg -> pwl
    initFields(pw2,pw);
#
    // encode decode
    if (integer_mode==PWMode::Qc) {
      // here assume a forward model and offset0=0 -> just change scale
#if TCH_PWMODEL_ATF_DOMAIN1
      for(int k=0;k<(int)pw2.lines.size();++k) pw2.lines[k].scale=pw2.lines[k].scale; // domainId=1: code scale, domainId=10 code 1/scale
#else
      for(int k=0;k<(int)pw2.lines.size();++k) pw2.lines[k].scale=1.f/pw2.lines[k].scale; // domainId=1: code scale, domainId=10 code 1/scale
#endif
    }
    toInteger(pw2);
    Tch::HDRConvertATFns::toFloat(pw2);
    if (integer_mode==PWMode::Qc) {
      // here assume a forward model and offset0=0 -> just change scale
#if TCH_PWMODEL_ATF_DOMAIN1
      for(int k=0;k<(int)pw2.lines.size();++k) pw2.lines[k].scale=pw2.lines[k].scale; // domainId=1: code scale, domainId=10 code 1/scale
#else
      for(int k=0;k<(int)pw2.lines.size();++k) pw2.lines[k].scale=1.f/pw2.lines[k].scale; // domainId=1: code scale, domainId=10 code 1/scale
#endif
    }
    return pw2;
  }
#endif

namespace Integer {
  // NB: currently int64 just for checking

  int64_t getValue(int x,int numBitsVal) {
    assert(x < (1 << numBitsVal) );
    const int signMask=(1<<(numBitsVal-1));
    bool neg=false;
    if (x&signMask) { neg=true; x=(x&(signMask-1)); }

    if (neg) x=-x;
    return x;
  }

  vector<int64_t> getScale(const PWModel &pw,int quantizer) {
    vector<int64_t> v(pw.ilines.size());
    const int scaleQ=pw.hdr_recon_scale_bit_depth+pw.hdr_recon_scale_frac_bit_depth;
    const int kReduce=pw.hdr_recon_scale_frac_bit_depth-quantizer;
    if (kReduce>=0) {
      const int kHalf=(1<<kReduce)/2;
      for(int i=0;i<(int)v.size();++i) {
        v[i]=(getValue(pw.ilines[i].scale,scaleQ)+kHalf)>>kReduce;
      }
    } else {
      for(int i=0;i<(int)v.size();++i) {
        v[i]=(getValue(pw.ilines[i].scale,scaleQ))<<(-kReduce);
      }
    }
    return v;
  }

  vector<int64_t> getAtfRangeIn(const PWModel &pw,int quantizer) {
    vector<int64_t> v(pw.ilines.size()+1);
    v[0]=0;
    const int deltaQ=pw.hdr_recon_offset_bit_depth+pw.hdr_recon_offset_frac_bit_depth;
    const int kReduce=(pw.hdr_recon_offset_frac_bit_depth-quantizer);
    const int kHalf=(1<<kReduce)/2;
    for(int k=1;k<(int)v.size();++k) {
      v[k]=v[k-1]+((getValue(pw.ilines[k-1].delta,deltaQ)+kHalf)>>kReduce);
    }
    return v;
  }

  vector<int64_t> getAtfRangeOut(const PWModel &pw,int quantizer) {
    vector<int64_t> v(pw.ilines.size()+1);
    const int scaleQ=pw.hdr_recon_scale_bit_depth+pw.hdr_recon_scale_frac_bit_depth;
    const int deltaQ=pw.hdr_recon_offset_bit_depth+pw.hdr_recon_offset_frac_bit_depth;

    const int kReduce1=(pw.hdr_recon_offset_frac_bit_depth-quantizer);
    const int kHalf1=(1<<kReduce1)/2;

    // warning here 64 bits op, should be reduce by first reducing scale and delta
    const int kReduce2=(pw.hdr_recon_scale_frac_bit_depth+pw.hdr_recon_offset_frac_bit_depth-quantizer);
    const int kHalf2=(1<<kReduce2)/2;
    v[0]=((getValue(pw.ioffset0,deltaQ)+kHalf1)>>kReduce1);
    for(int k=1;k<(int)v.size();++k) {
      v[k]=v[k-1]+((getValue(pw.ilines[k-1].delta,deltaQ)*getValue(pw.ilines[k-1].scale,scaleQ)+kHalf2)>>kReduce2);

    }
    return v;
  }


  std::vector<int64_t> getAtfOffset2(const PWModel &pw,int in_quantizer,int out_quantizer) {
    const vector<int64_t> In=getAtfRangeIn(pw,in_quantizer);
    const vector<int64_t> Out=getAtfRangeOut(pw,out_quantizer);
    const int scaleQ=pw.hdr_recon_scale_bit_depth+pw.hdr_recon_scale_frac_bit_depth;
    vector<int64_t> v(pw.ilines.size());

    const int kReduce=(in_quantizer+pw.hdr_recon_scale_frac_bit_depth-out_quantizer);
    const int kHalf=(1<<kReduce)/2;
    for(int k=0;k<(int)v.size();++k) {
      v[k]=Out[k+1]-((In[k+1]*getValue(pw.ilines[k].scale,scaleQ)+kHalf)>>kReduce);
    }
    return v;
  }

  std::vector<int> computeLut(const Tch::PWModel &pw, int quantizerInput,int quantizerLut) {
    assert(pw.mode==PWMode::Tch);
    const int numRanges = (int)pw.ilines.size(); // getAtfNumberRanges();
    const int kQuantizerScale=16;
    const vector<int64_t> pOffset   = getAtfOffset2(pw,quantizerInput,quantizerLut);
    const vector<int64_t> pAtfRange = getAtfRangeIn(pw,quantizerInput);
    const vector<int64_t> pScale=getScale(pw,kQuantizerScale);

    const int lut_size=(1<<quantizerInput);
    const int kReduce=(quantizerInput+kQuantizerScale-quantizerLut);
    const int kHalf=(1<<kReduce)/2;
    vector<int> v(lut_size);
    for (int i = 0; i < lut_size; i++) {
      bool skip = 0;
      const int x=i;
      for (int j = 0; j < numRanges && skip == 0; j++)
      {
        if (x <= pAtfRange[j + 1])
        {
          v[i] =(int) (((x *pScale[j]  + kHalf)>>kReduce)+pOffset[j]);
          skip = 1;
        }
      }
      if (skip == 0) {
        v[i] = (int)(((pAtfRange[numRanges] * pScale[numRanges - 1]+kHalf)>>kReduce)+pOffset[numRanges - 1]);
      }
    }
    return v;
  }
}

  Tch::PWModel lut2PWL(const std::vector<float> v, int nb_interval) {

    std::vector<int> nn;
    Tch::PWModel pw=Tch::LUT2PWLinearTch(v,nn,nb_interval,0.0001f);

    assert( TCH_ATF_OFFSET_BIT_DEPTH <= 10 ); //for 10bit content, no need to use offsets with bitdepth larger than 10

    pw.hdr_recon_scale_bit_depth        = TCH_ATF_SCALE_BIT_DEPTH-TCH_ATF_SCALE_FRAC_BIT_DEPTH; //for integer part of scale
    pw.hdr_recon_scale_frac_bit_depth   = TCH_ATF_SCALE_FRAC_BIT_DEPTH; //for fractional part of scale
    pw.hdr_recon_offset_bit_depth       = TCH_ATF_OFFSET_BIT_DEPTH-TCH_ATF_OFFSET_FRAC_BIT_DEPTH; //for integer part of offset
    pw.hdr_recon_offset_frac_bit_depth  = TCH_ATF_OFFSET_FRAC_BIT_DEPTH; //for fractional part of offset

    pw=Tch::refineIntegerModel(v,nn,pw,Tch::PWMode::Qc);

    if (fabs(pw.offset0)>1e-3) std::cerr<<"Pb offset0"<<std::endl;
    if ((int)pw.lines.size()<nb_interval) {
      const int ss=(int)pw.lines.size();
      pw.lines.resize(nb_interval);
      for(int k=ss;k<nb_interval;++k) {
        pw.lines[k].delta=0.f;
        pw.lines[k].scale=0.f;
      }
    }
    return pw;
  }  
}

#endif
#endif