/* Technicolor Confidential - Trade Secret
*/

/************* COPYRIGHT AND CONFIDENTIALITY INFORMATION *********
Copyright (c) 2015 Technicolor R&D France All Rights Reserved This program
contains proprietary information which is a trade secret/business secret of
Technicolor R&D France and is protected, even if unpublished, under applicable
Copyright laws (including French droit d'auteur). Recipient is to retain this
program in confidence and is not permitted to use or make copies thereof other
than as permitted in a written agreement with Technicolor R&D France unless
otherwise expressly allowed by applicable laws or by Technicolor R&D France
under express agreement.
*************************************************************************/

/*!
*************************************************************************************
* \file CrossPlaneReshaper.cpp
*
* \brief
*    CrossPlaneReshaper Class
*
* \author
*     -
*
*************************************************************************************
*/

#include <iostream>
#include <vector>
#include <cassert>
#include <algorithm>
#include <iostream>
#include <limits>
#include <sstream>
#include <fstream>
#include <cmath>
#include "ColorSpaceMatrix.H"
#include "Global.H"
#include "CrossPlaneReshaper.H"
#include "CrossPlaneFunctions.H"
#if TCH_CROSSPLANE_CRCB
//-----------------------------------------------------------------------------
// Include headers
using namespace Tch;
using namespace std;

namespace {



  struct YUV { // integer non linear YUV (aka Y'CbCr) in [0..1023]
    int Y, U, V;
    int operator[](int i) const { return (i==0)?Y:(i==1?U:V); }
    int &operator[](int i) { return (i==0)?Y:(i==1?U:V); }
  };

  inline YUVf convert_RGB2YUV(const RGBf p1,const double (*const m)[3]) { // to check
    YUVf p;
    for(int i=0;i<3;++i) p[i]=(float)(m[i][0]*p1[0]+m[i][1]*p1[1]+m[i][2]*p1[2]);
    return p;
  }

  namespace Constants{
    // constants
    const int kNbBits = 10;                    // nb bits for HDR/SDR
    const int kYMaxInteger = (1 << kNbBits) - 1;    // max value of Y U or V in integer (typically 1023 for 10 bits)
    const float kYValueMax = float(1 << kNbBits); // max floating value of Y denormalized (typically 1024.f for 10 bits)
    const int kNbIntervalPWModel=16;
  }

  const float peakContent=4000.f;

  // define pixel based color/space transformations and others
  template<int N> int clip(int x) { const int maxi=(1<<N)-1; return (x<0)?0:((x>=maxi)?maxi:x); }


  inline int getYColocU16(const Frame& frame, int xY, int yY)
  {
#if (TCH_AVERAGE_Y_COLOC_MODE == 1)
    int ySumInt = 0;
#endif
    int yColoc = 0;
#if TCH_AVERAGE_Y_COLOC_MODE == 0
    const int i = yY*frame.m_width[0] + xY; //left-up
    yColoc = frame.m_ui16Comp[0][i];
#elif TCH_AVERAGE_Y_COLOC_MODE == 1
    for (int y = 0; y < 2; y++)
    {
      for (int x = 0; x < 2; x++)
      {
        const int i = (yY + y)*frame.m_width[0] + (xY + x);
        ySumInt += frame.m_ui16Comp[0][i];
      }
    }
    yColoc = (ySumInt + 2 )/ 4;
#else
    assert(0);
    exit(0);
#endif
    return yColoc;
  }

  template<typename T> T toFullRange(T p, T weight, T offset) { return (weight * (p - offset)); }

  inline int toFullRange(int p, ::SampleRange sampleRange)
  {
    if (sampleRange == SR_FULL)
    {
      return p;
    }
    else if (sampleRange == SR_STANDARD)
    {
      assert(sampleRange == SR_STANDARD);
      int offset = (16 * (1 << (Constants::kNbBits - 8)));
      int weight = ((1 << 8) /* 219.0f*/);
      return clip<Constants::kNbBits>(toFullRange<int>(p, weight, offset) / 219);
    }
    else
    {
      assert(sampleRange == SR_RESTRICTED);

      int offset = (1 << (Constants::kNbBits - 8));
      int weight = (1 << Constants::kNbBits) /* (float)((1 << kNbBits) - (1 << (kNbBits - 7)))*/;
      return  clip<Constants::kNbBits>(toFullRange<int>(p, weight, offset) / ((1 << Constants::kNbBits) - (1 << (Constants::kNbBits - 7))));
    }
  }

  template<int N> int toInteger(float p) {
#if TCH_FLOAT_TO_INT_BALANCED
    return clip<N>((int)p);
#else
    return clip<N>((int)(p + 0.5f));
#endif
  }

  inline YUV toInteger(YUVf p) {
    YUV p2;
    p2.Y = toInteger<Constants::kNbBits>(p.Y);
    p2.U = toInteger<Constants::kNbBits>(p.U + Constants::kYValueMax*0.5f);
    p2.V = toInteger<Constants::kNbBits>(p.V + Constants::kYValueMax*0.5f);
    return p2;
  }
}

namespace Tch {
  // Constants
  using namespace std;
  //-----------------------------------------------------------------------------
  // Private methods
  //-----------------------------------------------------------------------------
  void HDRToolsFrameDataToTchDataDown(const Frame &framein, std::vector<Tch::RGBf> &v) {
    assert(framein.m_isFloat);
    assert(framein.m_format.m_chromaFormat==CF_444);
    assert(framein.m_colorSpace==CM_RGB);
    int width = framein.m_width[0] >> 1;

    for(int k=0;k<(int)v.size();++k) {
      int new_line = k / width;
      int new_col =  k - new_line * width;

      v[k].R=(framein.m_floatComp[R_COMP][(width << 1)*(new_line << 1) + (new_col <<1)]);
      v[k].G=(framein.m_floatComp[G_COMP][(width << 1)*(new_line << 1) + (new_col <<1)]);
      v[k].B=(framein.m_floatComp[B_COMP][(width << 1)*(new_line << 1) + (new_col <<1)]);
    }
  }

  void HDRToolsIntYUVFrameDataToTchData420Down(const Frame &YUVframein, std::vector<Tch::YUVf> &v) 
	{
    assert(YUVframein.m_format.m_chromaFormat==CF_420);
    int width = YUVframein.m_width[0] >> 1;
    for(int k=0;k<(int)v.size();++k) {
      int new_line = k / width;
      int new_col =  k - new_line * width;
      v[k].Y=YUVframein.m_ui16Comp[Y_COMP][(width << 1) * (new_line << 1)+ (new_col <<1)];
      v[k].U=YUVframein.m_ui16Comp[U_COMP][k];
      v[k].V=YUVframein.m_ui16Comp[V_COMP][k];
    }
  }

  static float evaluateFloatRepn(int x, int const numBitsVal, int const numBitsFrac)
  {
    assert(x < (1 << numBitsVal) );
    const int signMask=(1<<(numBitsVal-1));
    bool neg=false;
    if (x&signMask) { neg=true; x=(x&(signMask-1)); }

    int intPart = x >> numBitsFrac;
    int fracPart = x - (intPart << numBitsFrac);
    float retVal = (float)intPart + (float)fracPart / (float)(1 << numBitsFrac);
    if (neg) retVal=-retVal;
    return retVal;
  }


  static int evaluateIntRepn(float const x, int const numBitsVal, int const numBitsFrac)
  {
    int     nbInt=numBitsVal-numBitsFrac;
    int     maxi= (1<<nbInt)/2 - 1;//   /2 to manage negative values
    float   px = x;

    if (nbInt==0) nbInt=1;
    --nbInt; // sign
    if (maxi<=0) maxi=1;

    assert(px<=maxi && px>=-maxi); // need to use more bits to code integer value of px
    float fracVal;
    int signMask=0;
    if (px<0.f) { fracVal=-(px-(float)((int)px)); signMask=(1<<(numBitsVal-1)); px = -px; }   // always code the sign
    else         fracVal=px - (float)((int)px);
    int intRepnOfFrac = (int)((fracVal * (float)(1 << numBitsFrac)) + 0.5f);
    int intPart=(int)px;
    uint32_t retVal = (intPart<< numBitsFrac) + intRepnOfFrac;

    retVal|=signMask;
    assert(retVal < (1u << numBitsVal));
    return retVal;
  }

  
  //-----------------------------------------------------------------------------
  // Public methods
  //-----------------------------------------------------------------------------
  CrossPlaneReshaper::CrossPlaneReshaper()
  {
    m_betaU.resize(Constants::kYMaxInteger + 1);
    m_betaV.resize(Constants::kYMaxInteger + 1);

    m_denomU.resize(m_betaU.size());
    m_denomV.resize(m_betaV.size());
  }

  CrossPlaneReshaper::~CrossPlaneReshaper()
  {
  }

  void CrossPlaneReshaper::forward(Frame &out, const Frame &inp)
  {
    static const int kUVHalf=(1<<Constants::kNbBits)/2;


    assert(inp.m_chromaFormat == CF_420);

	for (int i = 0; i < inp.m_compSize[1]; i++) 
	{	
    const int Y =  getYColocU16(inp, (i%inp.m_width[1]) * 2, (i / inp.m_width[1]) * 2);
    const float beta0u = (float)m_forwardChromaReshapingLUT[0][Y] / m_betaLutDyn;
    const int U = inp.m_ui16Comp[1][i] - kUVHalf;
    const float Ucor = ( U / beta0u) + kUVHalf;
    out.m_ui16Comp[1][i] = std::max(0,std::min((1<<Constants::kNbBits)-1,(int) ( ((float)Ucor + .5))));

    const float beta0v = (float)m_forwardChromaReshapingLUT[1][Y] / m_betaLutDyn;
    const int V = inp.m_ui16Comp[2][i] - kUVHalf;
    const float Vcor = ( V / beta0v) + kUVHalf;
    out.m_ui16Comp[2][i] = std::max(0,std::min((1<<Constants::kNbBits)-1,(int) ((float) (Vcor + .5))));
	}
}

void CrossPlaneReshaper::compute_beta0_yuv_ref(std::vector<RGBf> &samplesRGB, std::vector<YUVf> &samplesYUV, std::vector<int> &nn, SampleRange range)
{
  YUV yuv_sdr_nc;
  const double (* const matrixRGB2YUV)[3]=FWD_TRANSFORM[CTF_RGB2020_2_YUV2020];
  nn.clear();
  nn.resize(m_betaU.size(),0);

  m_denomU.assign(m_denomU.size(), 0);
  m_denomV.assign(m_denomV.size(), 0);

#ifdef MODE1_UPDATE
  m_betaU.assign(m_denomU.size(), 0.f);
  m_betaV.assign(m_denomV.size(), 0.f);
#else
  m_betaU.assign(m_denomU.size(), -1.f);
  m_betaV.assign(m_denomV.size(), -1.f);
#endif

	int     bitDepthL = (int)(log((double)MAX_LUMA_RESHAPING_LUT_SIZE)/log(2.)+.5);
	float   offSR = 16.0 * (1<<(bitDepthL-8));
	float   scaleSR = 219. * (1<<(bitDepthL-8));
	float   offSR_C = 128.0 * (1<<(bitDepthL-8));
	float   scaleSR_C = 224. * (1<<(bitDepthL-8));

  int sliceStep = TCH_BETA_COMPUTE_STEP;
  float   Ymean = 0.;

  for (int k = 0; k < (int)samplesRGB.size(); k++) {
    //compute l_sdr non corrected
    const float L_unormalized = (float) (matrixRGB2YUV[0][0] * samplesRGB[k].R + matrixRGB2YUV[0][1] * samplesRGB[k].G + matrixRGB2YUV[0][2] * samplesRGB[k].B); // for Y only, we can apply the same coef to find the linear Y
    Ymean += L_unormalized;

    if ( L_unormalized > TCH_LUMINANCE_MIN )
    {
      yuv_sdr_nc.Y = (int) samplesYUV[k].Y;
      yuv_sdr_nc.U = (int) samplesYUV[k].U;
      yuv_sdr_nc.V = (int) samplesYUV[k].V;

      float YfullRange = (range == SR_FULL) ? (yuv_sdr_nc.Y) : std::max(0.,std::min(1023.,( 1023. * (yuv_sdr_nc.Y-offSR) / scaleSR )));
      float l_sdr = pow(YfullRange/1023.f, 1.f/0.45f);

      //compure ratio
      float ratio = l_sdr / L_unormalized ;

      //compure yuv ref
      RGBf rgb_sdr;
#if MODE1_UPDATE
      rgb_sdr.R = std::max(0.f,std::min(1.f,ratio * samplesRGB[k].R));
      rgb_sdr.G = std::max(0.f,std::min(1.f,ratio * samplesRGB[k].G));
      rgb_sdr.B = std::max(0.f,std::min(1.f,ratio * samplesRGB[k].B));
#else
      rgb_sdr.R = ratio * samplesRGB[k].R;
      rgb_sdr.G = ratio * samplesRGB[k].G;
      rgb_sdr.B = ratio * samplesRGB[k].B;
#endif

      rgb_sdr.R = pow(rgb_sdr.R, 0.45f) * 1023.f;
      rgb_sdr.G = pow(rgb_sdr.G, 0.45f) * 1023.f;
      rgb_sdr.B = pow(rgb_sdr.B, 0.45f) * 1023.f;

      YUVf yuvp = convert_RGB2YUV(rgb_sdr,matrixRGB2YUV); //* by matrix RGB2020 to YUV 2020

      if (range == SR_STANDARD)
      {
        yuvp.Y = (offSR + scaleSR*yuvp.Y/1023.);
#if MODE1_UPDATE
        yuvp.U = (float)std::max(0.,std::min(1023., offSR_C + scaleSR_C*yuvp.U/1023.));
        yuvp.V = (float)std::max(0.,std::min(1023., offSR_C + scaleSR_C*yuvp.V/1023.));
#else
        yuvp.U = std::max(0.,std::min(1023., offSR_C + scaleSR*yuvp.U/1023.));
        yuvp.V = std::max(0.,std::min(1023., offSR_C + scaleSR*yuvp.V/1023.));
#endif
      }
      else
      {
        yuvp.U = std::max(0.,std::min(1023., 512. + yuvp.U));
        yuvp.V = std::max(0.,std::min(1023., 512. + yuvp.V));
      }

      int sliceIdx = ((yuv_sdr_nc.Y / sliceStep)+1)*sliceStep;
      sliceIdx = std::min<int>(m_betaU.size()-1, sliceIdx);

#ifdef MODE1_UPDATE
      m_betaU[sliceIdx] = (float)( m_betaU[sliceIdx] + (yuvp.U-512)*(yuv_sdr_nc.U-512) );
      m_betaV[sliceIdx] = (float)( m_betaV[sliceIdx] + (yuvp.V-512)*(yuv_sdr_nc.V-512) );
#else
      m_betaU[sliceIdx] = (float)( std::max<float>(0.f, m_betaU[sliceIdx]) + ( (yuvp.U-512)*(yuv_sdr_nc.U-512) ) );
      m_betaV[sliceIdx] = (float)( std::max<float>(0.f, m_betaV[sliceIdx]) + ( (yuvp.V-512)*(yuv_sdr_nc.V-512) ) );
#endif

      m_denomU[sliceIdx] += ((yuv_sdr_nc.U-512) * (yuv_sdr_nc.U-512));
      m_denomV[sliceIdx] += ((yuv_sdr_nc.V-512) * (yuv_sdr_nc.V-512));
      ++nn[sliceIdx];
    }
  }

  Ymean /= samplesRGB.size();

  int last_y = -1;
  for (int i = 0; i < (int)m_betaU.size(); i++)
  {
//    if ( betaU[i] != -1 && betaU[i] != 0 && betaV[i] != -1 && betaV[i] != 0 && nn[i] > 0.001*samplesRGB.size() )
    if ( nn[i] > 0.001*samplesRGB.size() && m_betaU[i] > 0 && m_betaV[i] > 0 )
    {
      m_betaU[i] = m_denomU[i] / m_betaU[i];
      m_betaV[i] = m_denomV[i] / m_betaV[i];

      for (int j = last_y+1; j < i; j++){
        if (last_y == -1)
        {
          m_betaU[j] = m_betaU[i];
          m_betaV[j] = m_betaV[i];
        }
        else
        {
          m_betaU[j] = (float)(j - last_y) / (float)(i - last_y) * (m_betaU[i] - m_betaU[last_y]) + m_betaU[last_y] ;
          m_betaV[j] = (float)(j - last_y) / (float)(i - last_y) * (m_betaV[i] - m_betaV[last_y]) + m_betaV[last_y] ;
        }
      }
      last_y = i ;
    }
    else
    {
      m_betaU[i] = -1;
      m_betaV[i] = -1;
    }
  }

  for (int i = last_y; i < (int)m_betaU.size(); i++)
  {
    m_betaU[i] = m_betaU[last_y-1];
    m_betaV[i] = m_betaV[last_y-1];
  }
 
    LUT_Beta0s.first.data.resize(m_betaU.size());
    LUT_Beta0s.second.data.resize(m_betaU.size());

  for (int k = 0; k<(int)m_betaU.size(); ++k) {
    LUT_Beta0s.first.data[k] = (float) m_betaU[k] ;
    LUT_Beta0s.second.data[k] = (float) m_betaV[k];
  }
  LUT_Beta0s.first.linear_quantizer = (float)(LUT_Beta0s.first.data.size()) / Constants::kYValueMax;
  LUT_Beta0s.second.linear_quantizer = (float)(LUT_Beta0s.second.data.size()) / Constants::kYValueMax;
}

#if ARRS_MODEL_OPTIMIZATION
void CrossPlaneReshaper::setCrossCompLUT(float *lut, int lutSize, int chromaIndex)
{
  if(chromaIndex == 0)
  {
  LUT_Beta0s.first.data.resize(lutSize);
    for (int k = 0; k<lutSize; ++k) 
    LUT_Beta0s.first.data[k] = (float) lut[k] ;
  }
  else //if(chromaIndex == 1)
  {
    LUT_Beta0s.second.data.resize(lutSize);
    for (int k = 0; k<lutSize; ++k)
    LUT_Beta0s.second.data[k] = (float) lut[k];
  }
  return;
}

void CrossPlaneReshaper::getForwardCrossScaleLUT(int *lut, int lutSize, int chromaIndex)
{
  
  for (int k = 0; k<lutSize; ++k) 
    lut[k]=(int) (CHROMA_INV_LUt_SCALE * m_betaLutDyn/(float)m_forwardChromaReshapingLUT[chromaIndex][k]);
  
  return;
}
#endif

void CrossPlaneReshaper::setParamsTCHYATF(Tch::PWModel &pw)
{
  float     scaleValue, DeltaValue, globalOffset;
  int       flagEqualRanges = pw.hdr_recon_equal_ranges_flag;
  int       m_atfDomainId = 1;
  float     m_offset2 = (float)pw.ioffset0;  
  int       m_atfNumberRanges = pw.hdr_recon_num_ranges() ;

  globalOffset = evaluateFloatRepn(evaluateIntRepn(pw.offset0 / (1<<TCH_GLOBAL_OFFSET_SHIFT) , TCH_ATF_OFFSET_BIT_DEPTH, TCH_ATF_OFFSET_FRAC_BIT_DEPTH), TCH_ATF_OFFSET_BIT_DEPTH, TCH_ATF_OFFSET_FRAC_BIT_DEPTH);
  pw.offset0 = globalOffset * (1<<TCH_GLOBAL_OFFSET_SHIFT);

  for (int i = 0; i < m_atfNumberRanges; i++)
  {
    scaleValue = evaluateFloatRepn(evaluateIntRepn(pw.lines[i].scale , TCH_ATF_SCALE_BIT_DEPTH, TCH_ATF_SCALE_FRAC_BIT_DEPTH), TCH_ATF_SCALE_BIT_DEPTH, TCH_ATF_SCALE_FRAC_BIT_DEPTH);
    pw.lines[i].scale = scaleValue;
  }
  if (flagEqualRanges == 0)
  {
    for (int i = 0; i < m_atfNumberRanges; i++)
    {
      DeltaValue = evaluateFloatRepn(evaluateIntRepn(pw.lines[i].delta , TCH_ATF_OFFSET_BIT_DEPTH, TCH_ATF_OFFSET_FRAC_BIT_DEPTH), TCH_ATF_OFFSET_BIT_DEPTH, TCH_ATF_OFFSET_FRAC_BIT_DEPTH);
      pw.lines[i].delta = DeltaValue;
    }
  }
}

void CrossPlaneReshaper::encode_beta0_lut()
{
  double meta_max = (double)(MAX_LUMA_RESHAPING_LUT_SIZE-1);
  const int nb_interval = TCH_NB_PIECES_PWL;

  {

    //PWL modeling
    float     minValU = LUT_Beta0s.first.data[0];
    float     minValV = LUT_Beta0s.second.data[0];
    for (int i=0;i<LUT_Beta0s.first.data.size();i++)
    {
      LUT_Beta0s.first.data[i] -= minValU;
      LUT_Beta0s.second.data[i] -= minValV;
    }

    pwlU = Tch::lut2PWL(LUT_Beta0s.first.data, nb_interval);
    pwlV = Tch::lut2PWL(LUT_Beta0s.second.data, nb_interval);

    //generate PWL and regenerate LUT (with non-quantized ATF coefficients)
    vector<float>             AtfLutU = Tch::computeLut(pwlU,LUT_Beta0s.first.data.size());
    vector<float>             AtfLutV = Tch::computeLut(pwlV,LUT_Beta0s.second.data.size());

    pwlU.offset0 = minValU;
    pwlV.offset0 = minValV;
    for (int i=0;i<MAX_LUMA_RESHAPING_LUT_SIZE;i++)
    {
      AtfLutU[i] += pwlU.offset0;
      AtfLutV[i] += pwlV.offset0;
    }

    //quantization of PWL coefs
    setParamsTCHYATF(pwlU);
    setParamsTCHYATF(pwlV);

    generateRangesAndOffsets(pwlU, 0);
    generateRangesAndOffsets(pwlV, 1);
  }
}
void CrossPlaneReshaper::writeCbCrATFCrossPlane(char *filename, int POC, int reshaperRangeId, int reshapingMode, int *offsetArray, int numParams)
{
  FILE    *atfParamFile = fopen(filename, "a+");
  int     m_atfDomainId = 1;
  int	  l_atfFlagEqualRanges[2] =	{ 0, 0 };
  int     tmp;

  //output metadata: POC always starts from zero to be aligned with POC at encoder.
    fprintf(atfParamFile, "%d ", POC);
	fprintf(atfParamFile, "%d ", reshaperRangeId);
    fprintf(atfParamFile, "%d ", m_atfDomainId);
	fprintf(atfParamFile, "%d ", reshapingMode); //chroma Mode = 0 means auto-plane 
	fprintf(atfParamFile, "%d ", pwlU.hdr_recon_scale_bit_depth); //reshape_scale_int_bit_depth
	fprintf(atfParamFile, "%d ", pwlU.hdr_recon_offset_bit_depth); //reshape_offset_int_bit_depth
	fprintf(atfParamFile, "%d ", pwlU.hdr_recon_scale_frac_bit_depth); //reshape_scale_frac_bit_depth
	fprintf(atfParamFile, "%d ", pwlU.hdr_recon_offset_frac_bit_depth); //reshape_offset_frac_bit_depth
	fprintf(atfParamFile, "%d ", 1); //reshape_negative_scale
    assert((numParams==1) || (numParams==2) );
    fprintf(atfParamFile, "%d ", numParams); //number of chroma reshaping parameters. 

  //Cb
  fprintf(atfParamFile, "%d ", pwlU.lines.size());
  fprintf(atfParamFile, "%d ", l_atfFlagEqualRanges[0]);
  
  tmp = evaluateIntRepn(pwlU.offset0 / (1<<TCH_GLOBAL_OFFSET_SHIFT), TCH_ATF_OFFSET_BIT_DEPTH + TCH_ATF_OFFSET_FRAC_BIT_DEPTH, TCH_ATF_OFFSET_FRAC_BIT_DEPTH);// Zero offset value
  fprintf(atfParamFile, "%d ", tmp);
  // Write scale value for each bin
  for (int i = 0; i < pwlU.lines.size(); i++)
  {
    tmp = evaluateIntRepn(pwlU.lines[i].scale, TCH_ATF_SCALE_BIT_DEPTH, TCH_ATF_SCALE_FRAC_BIT_DEPTH);
    fprintf(atfParamFile, "%d ", tmp);
  }
  if (l_atfFlagEqualRanges[0] == 0)
  {
    for (int i = 0; i < pwlU.lines.size(); i++)
    {
      tmp = evaluateIntRepn(pwlU.lines[i].delta, TCH_ATF_OFFSET_BIT_DEPTH, TCH_ATF_OFFSET_FRAC_BIT_DEPTH);
      fprintf(atfParamFile, "%d ", tmp);
    }
  }

  fprintf(atfParamFile, "%d ", offsetArray[0]); // reshap_offset1
  fprintf(atfParamFile, "%d ", offsetArray[1]); // reshap_offset2
  if(numParams==2)
  {
  //Cr
  fprintf(atfParamFile, "%d ", pwlV.lines.size());
  fprintf(atfParamFile, "%d ", l_atfFlagEqualRanges[1]);
  tmp = evaluateIntRepn(pwlV.offset0 / (1<<TCH_GLOBAL_OFFSET_SHIFT), TCH_ATF_OFFSET_BIT_DEPTH + TCH_ATF_OFFSET_FRAC_BIT_DEPTH, TCH_ATF_OFFSET_FRAC_BIT_DEPTH);// Zero offset value
  fprintf(atfParamFile, "%d ", tmp);
  // Write scale value for each bin
  for (int i = 0; i < pwlV.lines.size(); i++)
  {
    tmp = evaluateIntRepn(pwlV.lines[i].scale, TCH_ATF_SCALE_BIT_DEPTH, TCH_ATF_SCALE_FRAC_BIT_DEPTH);
    fprintf(atfParamFile, "%d ", tmp);
  }
  if (l_atfFlagEqualRanges[1] == 0)
  {
    for (int i = 0; i < pwlV.lines.size(); i++)
    {
      tmp = evaluateIntRepn(pwlV.lines[i].delta, TCH_ATF_OFFSET_BIT_DEPTH, TCH_ATF_OFFSET_FRAC_BIT_DEPTH);
      fprintf(atfParamFile, "%d ", tmp);
    }
  }
  fprintf(atfParamFile, "%d ", offsetArray[2]); // reshap_offset1
  fprintf(atfParamFile, "%d ", offsetArray[3]); // reshap_offset2
  }

  fprintf(atfParamFile, "\n");
  fclose (atfParamFile);
}


void CrossPlaneReshaper::process(const Frame &RGB_framein, const Frame &Reshaped_YUV_frame ) {

  m_imageRGB_down.resize(Reshaped_YUV_frame.m_compSize[0] >> 2);
  m_image_reshaped_420YUV.resize(RGB_framein.m_compSize[0] >> 2);


  HDRToolsFrameDataToTchDataDown(RGB_framein, m_imageRGB_down);
  HDRToolsIntYUVFrameDataToTchData420Down(Reshaped_YUV_frame, m_image_reshaped_420YUV);  
  compute_beta0_yuv_ref( m_imageRGB_down, m_image_reshaped_420YUV,  m_nn, Reshaped_YUV_frame.m_sampleRange); 
}

void CrossPlaneReshaper::generateRangesAndOffsets(Tch::PWModel &pw, int c) 
{
  std::vector<int>    inputRanges, outputRanges, offsetValue;
  int                 numRanges = pw.lines.size();

  int     bitDepthL = 10;
  int     deltaBD1 = TCH_DEFAULT_PRECISION + pw.hdr_recon_offset_frac_bit_depth - bitDepthL;

  m_betaLutDyn = 1 << (pw.hdr_recon_scale_frac_bit_depth + pw.hdr_recon_offset_frac_bit_depth + TCH_DEFAULT_PRECISION);

  inputRanges.resize(numRanges+1);
  outputRanges.resize(numRanges+1); 

  for(int i = 0; i <= numRanges; i++) 
  {
    if(i == 0) 
    {
      int   globalOffset = (int)( pw.offset0 * ( 1 << (pw.hdr_recon_offset_bit_depth + pw.hdr_recon_offset_frac_bit_depth - TCH_GLOBAL_OFFSET_SHIFT) ) ); //quantized value 
      outputRanges[0] = globalOffset << (TCH_GLOBAL_OFFSET_SHIFT + pw.hdr_recon_scale_frac_bit_depth + TCH_DEFAULT_PRECISION);
      inputRanges[0] = 0;
    } 
    else 
    {
      int   delta = (int)( (1<<(pw.hdr_recon_offset_frac_bit_depth+TCH_DEFAULT_PRECISION)) * pw.lines[i-1].delta ); //quantized value
      int   scale = (int)( (1<<(pw.hdr_recon_scale_frac_bit_depth)) * pw.lines[i-1].scale ); //quantized value
      inputRanges[i]  = inputRanges[i-1] + delta;
      outputRanges[i] = outputRanges[i-1] + delta * scale;
    }
  }

  offsetValue.resize(numRanges);
  for(int i = 0; i < numRanges; i++) 
  {
    int   scale = (int)( (1<<(pw.hdr_recon_scale_frac_bit_depth)) * pw.lines[i].scale ); //quantized value
    offsetValue[i] = outputRanges[i] - (inputRanges[i] * scale);
  }

  for(int i=0; i < MAX_LUMA_RESHAPING_LUT_SIZE; i++)
  {
      //find PWL index
      int   idx = 0;
      int   ii = i << deltaBD1;

      if ( ii <= inputRanges[0] )
        idx = 0;
      else if ( ii > inputRanges[numRanges] )
        idx = numRanges - 1;
      else
      {
        for (idx=0; idx<numRanges; idx++)
        {
          if (ii > inputRanges[idx] && ii <=inputRanges[idx+1])
            break;
        }
      }

      int   scale = (int)( (1<<(pw.hdr_recon_scale_frac_bit_depth)) * pw.lines[idx].scale ); //quantized value
      int   val1 = ( i * scale ) << deltaBD1;
      int   val2 = offsetValue[idx];
      int   val = val1 + val2;

      m_forwardChromaReshapingLUT[c][i] = val1 + val2;
  }
}

//-----------------------------------------------------------------------------
// End of file
//-----------------------------------------------------------------------------
}
#endif